<?php

namespace HCM\FPA\Controllers;

use HCM\FPA\Services\ParametroSindicatoService;
use HCM\Util\HCMParameters;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response\Notification;

class ParametroSindicatoController extends Crud {

    protected $dataSourceName = 'fpa_paramsindicato_get';
    protected $parametroSindicatoService;
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO','P_DTMESCOMPETENC');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, ParametroSindicatoService $parametroSindicatoService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->parametroSindicatoService = $parametroSindicatoService;
        parent::__construct($dataSourceManager);
    }

    public function getParametroSindicato(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function replicaParametroSindicato(DTO\Request\DataSet $request, DTO\Response $response) {
        $row=$request->getDataSet()->getRows();
        $row=$row[0];
        try {
            foreach ($row["NRESTRUTURAM"]as $nrestruturam) {
                
                if($this->parametroSindicatoService->replicaParametroSindicato($row, $nrestruturam, $nmestruturam)){
                    $response->addNotification(new Notification("Parâmetro replicado com sucesso para a estrutura $nmestruturam", Notification::TYPE_SUCCESS));
                }else{
                    $response->addNotification(new Notification("Parâmetro existente para a estrutura $nmestruturam",Notification::TYPE_ERROR));
                }
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function setParametroSindicato(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = 'fpa_paramsindicato';
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRSINDICATO', $row)) {
                    if ($row['NRSINDICATO'] === '') {
                        $row['NRSINDICATO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_PARAMSINDICATO', 1, 12, true); 
                    }
                } else {
                    $row['NRSINDICATO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_PARAMSINDICATO', 1, 12, true); 
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteParametroSindicato(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = 'fpa_paramsindicato';
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 