<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Geral\Controllers\RelatoriosHcmController;
use HCM\Util\HCMParameters;
use HCM\Util\FPA;
use HCM\Util\Repositories;
use Doctrine\ORM\EntityManager;
use HCM\FPA\ObrigacaoLegal\SEFIP\PercorreSEFIP;

class SEFIPController extends Crud {

    protected $SEFIPService;
    protected $dataSourceNameGet = 'getSEFIP';
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC', 'P_CDOPERADOR');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, EntityManager $entityManager, RelatoriosHcmController $relatoriosHcmController){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->entityManager = $entityManager;
        $this->relatoriosHcmController = $relatoriosHcmController;
        parent::__construct($dataSourceManager);
    }

    public function getSEFIP (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); $rows = $dataSet->getRows();
        foreach($rows as $key => &$row){
            if($row['DSORDEM'] == 'LISTA_VINCULO'){
                $row['NRVINCULOM_LIST'] = explode(",", $row['DSINICIO']);
                foreach($row['NRVINCULOM_LIST'] as $chave => $value)
                    $row['NRVINCULOM_LIST'][$chave] = intval($value);
                
                $row['NMVINCULOM'] = explode(",", $row['NMVINCULOM']);
            }    
            
            if($row['DSORDEM'] == 'LISTA_ESTRUTURA'){
                $row['LIST_NRESTRUTLEGAL'] = explode(",", $row['DSINICIO']);
                foreach($row['LIST_NRESTRUTLEGAL'] as $chave => $value)
                    $row['LIST_NRESTRUTLEGAL'][$chave] = intval($value);
                
                $row['LIST_NMESTRUTLEGAL'] = explode(",", $row['NMESTRUTLEGAL']);
                
                if($row['DSFIM']){
                    $row['LIST_NRESTRUTGEREN'] = explode(",", $row['DSFIM']);
                    foreach($row['LIST_NRESTRUTGEREN'] as $chave => $value)
                        $row['LIST_NRESTRUTGEREN'][$chave] = intval($value);
                    
                    $row['LIST_NMESTRUTGEREN'] = explode(",", $row['NMESTRUTGEREN']);
                }
            }
            
            if($row['DSORDEM'] === "LISTA_NRESTRUTSIND"){
                if($row['DSFIM']) {
                    $row['LIST_NRESTRUTSIND'] = explode(',',$row['DSFIM']);
                    foreach($row['LIST_NRESTRUTSIND'] as $chave => $value)
                        $row['LIST_NRESTRUTSIND'][$chave] = intval($value);
                        
                    $row['LIST_NMESTRUTSIND'] = explode(",", $row['NMESTRUTSIND']);
                }
            }
        }
        $response->addDataSet(new DataSet($this->dataSourceNameGet, $rows));
    }
    
     public function setSEFIP(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "fpa_SEFIP";
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRFORMGFIP', $row)) {
                    if ($row['NRFORMGFIP'] === '') {
                        $row['NRFORMGFIP'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'FPA_FORMGFIP', 1, 12, true); 
                    }
                } else {
                    $row['NRFORMGFIP'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'FPA_FORMGFIP', 1, 12, true); 
                }
                if(!array_key_exists('DSINICIO', $row)){
                    $row['DSINICIO'] = null;
                }
    
                if(!array_key_exists('DSFIM', $row)){
                    $row['DSFIM'] = null;
                }
                
                // define os campos de descrição de início e fim com base no critério de seleção passado pela row
                if($row['DSORDEM'] === "LISTA_VINCULO"){
                    if(is_array($row['DSINICIO'])){
                        $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                    }
                }
                
                if($row['DSORDEM'] === "LISTA_ESTRUTURA"){
                    if(is_array($row['DSINICIO'])){
                        $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                    }
                    if(is_array($row['DSFIM']) && count($row['DSFIM']) > 0){
                        $row['DSFIM'] = implode(',',$row['DSFIM']);
                    }else{
                        if(is_array($row['DSFIM'])){
                            $row['DSFIM'] = null;
                        }
                    }
                }
                
                if($row['DSORDEM'] === "LISTA_NRESTRUTSIND"){
                    if(count($row['DSFIM']) > 0 && is_array($row['DSFIM'])){
                        $row['DSFIM'] = implode(',',$row['DSFIM']);
                    }else{
                        if(is_array($row['DSFIM'])){
                            $row['DSFIM'] = null;
                        }
                    }
                }
                
                $row['NRESTRUTRESPONS'] = isset($row['NRESTRUTRESPONS'])?$row['NRESTRUTRESPONS']:null;
                $row['NRHIERARTOMADOR'] = isset($row['NRHIERARTOMADOR'])?$row['NRHIERARTOMADOR']:null;
                $row['NRPESSOARESPONS'] = isset($row['NRPESSOARESPONS'])?$row['NRPESSOARESPONS']:null;
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteSEFIP(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $this->dataSourceName = "fpa_SEFIP";
            parent::delete($request, $response);
            $response->addNotification(new Notification("row(s) deleted.", Notification::TYPE_SUCCESS));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
    }
    
    public function geraSEFIP(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            ini_set('memory_limit', '2048M');
            set_time_limit(0);
            error_reporting(E_ERROR | E_PARSE);
            
            $dataset = $request->getDataSet()->getRows();
            $dataSet = $dataset[0];
            
            if ($dataSet['TP_PROCESSAMENTO'] == 'COMPETENCIA_13') {
                if (($dataSet['IDMODALIDADE'] != '1') && ($dataSet['IDMODALIDADE'] != '9')) {
                    $message = new \Zeedhi\Framework\DTO\Response\Message('Para a competência 13, a modalidade deve ser igual a 1 ou 9.', DTO\Response\Message::TYPE_ERROR);
                    $response->addMessage($message);
                    return;
                }
            }
            if ($dataSet['TP_PROCESSAMENTO'] != 'RETROCESSO') 
                $this->relatoriosHcmController->setRelatoriosHcmBackend(array("P_COMPETENCIA"=> $this->hcmParameters->getDtCompetencia()),'analitico_sefip', $request, $response);
            
            $percorreSEFIP = new PercorreSEFIP($this->entityManager, $this->hcmParameters, $dataSet);
            $resultado = $percorreSEFIP->percorre();

            if($resultado['ARQUIVO']){
                $row = array('NMARQUIVO' => FPA::getTempDirectory().'/'.$resultado['ARQUIVO']);
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', $row));
                $message = new \Zeedhi\Framework\DTO\Response\Message(
                    'finalizado sem erros.', DTO\Response\Message::TYPE_MESSAGE);
            }
            if($resultado['RETROCESSO']) {
                 $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Retrocedido com sucesso.", \Zeedhi\Framework\DTO\Response\Message::TYPE_MESSAGE));
            }
            
            if ($resultado['ABORT'] === true) {
                $message = $this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->findBy(array('nrcalculofolha' => $resultado['CALCULO'], 'nrorg' => $this->hcmParameters->getNrorgTrabalho(), 'dsorigmsg' => 'SEFIP'));
            }elseif ($resultado['ERRO'] === false) {
                $message = new \Zeedhi\Framework\DTO\Response\Message($dataSet['TP_PROCESSAMENTO'].' finalizado sem erros.', DTO\Response\Message::TYPE_MESSAGE);
            } else {
                $message = $this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->findBy(array('nrcalculofolha' => $resultado['CALCULO'], 'nrorg' => $this->hcmParameters->getNrorgTrabalho(), 'dsorigmsg' => 'SEFIP'));
            }
            
            $this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->limpaMsgCalculo($resultado['CALCULO'], $this->hcmParameters->getNrorgTrabalho(), 'SEFIP');  
            
            if($message instanceof \Zeedhi\Framework\DTO\Response\Message){
                $response->addMessage($message);
            } else {
                $row = array();
                foreach ($message as $mensagem) {
                    $row[] = array("NRVINCULOM" => $mensagem->getNrvinculom(), "NRMSGCALC" => $mensagem->getNrmsgcalc(), "DSDESCRICAO" => $mensagem->getDescricao(), "IDTIPOMSG" => $mensagem->getIdtipomsg());
                }
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('msgCalculo', $row));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
}