<?php
namespace HCM\FPA\Services;

use Doctrine\ORM\EntityManager;
use HCM\Util\DateUtil;
use HCM\FPA\Repository\DependenteVincRepository;
use HCM\Geral\Repository\PessoaRepository;
use HCM\Geral\Services\ParcnegocioService;
use HCM\Geral\Services\PessoaService;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;

class DependenteVincService {

    const TIPO_RELACIONAMENTO_FILHO       = 3;
    const TIPO_RELACIONAMENTO_TUTELADO    = 5;
    const TIPO_RELACIONAMENTO_PENSIONISTA = 6;

    const TIPODEPENDE_SALARIO_FAMILIA     = 1;
    const TIPODEPENDE_IMPOSTO_DE_RENDA    = 2;
    const TIPODEPENDE_PENSAO_JUDICIAL     = 3;
    

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, ParcnegocioService $parcnegocioService, PessoaService $pessoaService,
                                DependenteVincRepository $dependenteVincRepository, PessoaRepository $pessoaRepository) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->parcnegocioService = $parcnegocioService;
        $this->pessoaService = $pessoaService;
        $this->dependenteVincRepository = $dependenteVincRepository;
        $this->pessoaRepository = $pessoaRepository;
        $this->idadeMinCPF = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->buscavalor($this->hcmParameters->getNrorgTrabalho(), $this->hcmParameters->getNrorgPadrao(), $this->hcmParameters->getDtCompetencia(), 'IDADE_MIN_CPF_DEP');
        
        $this->fpaDepvinculo = Repositories::FPA_DEPVINCULO;
    }

    public function novo($nrorg, $nrpessoa, $nrtipodepende, $nrvinculom)
    {
        $depvinculo = new $this->fpaDepvinculo;

        $depvinculo->setNrorg($nrorg);
        $depvinculo->setNrpessoa($nrpessoa);
        $depvinculo->setNrtipodepende($nrtipodepende);
        $depvinculo->setNrvinculom($nrvinculom);
        $depvinculo->setNrdepvinculo(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_DEPVINCULO', 1, 12, true));

        $depvinculo->setDtinclusao(DateUtil::getDataAtual());
        $depvinculo->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $depvinculo->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $depvinculo->setDtultatu(DateUtil::getDataAtual());
        $depvinculo->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $depvinculo->setCdoperultatu($this->hcmParameters->getCdoperador());

        $this->entityManager->persist($depvinculo);
        $this->entityManager->flush($depvinculo);
        return $depvinculo;
    }

    public function verificaExisteDependencia($nrorg, $nrdepvinculo, $nrvinculom, $nrpessoa, $nrtipodepende, $isNew=false)
    {
        try {
            if ($isNew) {
                if ($this->existeDependencia($nrorg, $nrpessoa, $nrtipodepende, $nrvinculom) === true) {
                    throw new \Exception(Messages::HCM01000_FPA_DEPVINCULO_DEPEND_EXISTE_TIPODEPENDE, 10);
                }
            } else {
                $dependencias = $this->retornaDependenciasPorVinculo($nrvinculom, $nrorg);
                foreach ($dependencias as $depvinculo) {
                    if ($nrdepvinculo != $depvinculo->getNrdepvinculo()) { //Exceto o próprio registro que está sendo alterado
                        if ($depvinculo->getNrpessoa() == $nrpessoa && $depvinculo->getNrtipodepende() == $nrtipodepende) {
                            throw new \Exception(Messages::HCM01000_FPA_DEPVINCULO_DEPEND_EXISTE_TIPODEPENDE, 10);
                        }
                    }
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function existeDependencia($nrorg, $nrpessoa, $nrtipodepende, $nrvinculom)
    {
        try {
            $depvinculo = $this->entityManager->getRepository(Repositories::FPA_DEPVINCULO)->findOneBy(
                array('nrorg'=>$nrorg, 'nrpessoa'=> $nrpessoa, 'nrtipodepende'=>$nrtipodepende, 'nrvinculom'=>$nrvinculom)
            );
            return is_object($depvinculo) > 0;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function retornaDependenciasPorVinculo($nrvinculom, $nrorg)
    {
        try {
            $dependencias = $this->entityManager->getRepository(Repositories::FPA_DEPVINCULO)->findBy(
                array('nrorg'=>$nrorg, 'nrvinculom'=>$nrvinculom)
            );
            return $dependencias;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function realizaCadastroPessoaDependente($nrvinculom, $nrpessoavinc, $nmpessoa, $cdmunicipio, $sgestado, $cdpais, $nrnacionalid, $cdestacivil, $idsexopessoa,
                                                    $dtnascpessoa, $nrtiporelaciona, $dtfimrelacionamento, $cpfpessoa, $verificaCPF = false) {

        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $dtmescompetencia = $this->hcmParameters->getDtCompetencia();

        $idpessoafisica = 'S';
        $idinstituicao = 'N';
        $cdtipoinscricao = 'CPF';
        $cdtipoparceiro = 'PESSOA';

        try {
            $this->entityManager->beginTransaction();

            $pessoaVinc = $this->entityManager->getRepository(Repositories::GPE_PESSOA)->findOneBy(
                array('nrorg'=>$nrorg, 'nrpessoa'=>$nrpessoavinc)
            );
            $nrparcnegocio = $pessoaVinc->getNrparcnegocio();

            if ($dtnascpessoa){
                $dtnascpessoa = DateUtil::getDataDeString($dtnascpessoa, DateUtil::FORMATO_BRASILEIRO, true);
            }else{
                $dtnascpessoa = null;
            }    
            
            $parcnegocio = $this->parcnegocioService->novo(
                $nrorg, $nmpessoa, $nmpessoa, $cdtipoparceiro, $idpessoafisica, $idinstituicao, $cdtipoinscricao,
                '00000000000', $dtnascpessoa
            );

            $dtcompetencia = DateUtil::getDataDeString($dtmescompetencia, DateUtil::FORMATO_BRASILEIRO, true);
            
            $pessoa = $this->pessoaService->novo($nrorg, $parcnegocio->getNrparcnegocio(), $dtcompetencia, $nmpessoa, null,
                null, $dtnascpessoa, $idsexopessoa, null, $cpfpessoa, null, null, null, null, null, null, null, null, $cdestacivil, null, null,
                null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, $cdpais, $sgestado, $cdmunicipio,
                null, null, null, null, null, $nrnacionalid, null);

            // CADASTRO DE DEPENDENTE
            $this->parcnegocioService->novoRelacionaparc($nrorg, $nrtiporelaciona, $nrparcnegocio, $dtfimrelacionamento, $parcnegocio->getNrparcnegocio());

            $tipodepcadastro = array(3, 5, 6);
            if(in_array ($nrtiporelaciona , $tipodepcadastro)) {
                $dependentes = array();
                $dependentes["NRPARCDEPENDE"] = $parcnegocio->getNrparcnegocio();
                $dependentes["NRORG"] = $nrorg;
                $dependentes["NRTIPORELACIONA"] = $nrtiporelaciona;

                $dependentes = array($dependentes);
                $this->cadastroAutomaticoDependentes($nrorg, $nrvinculom, $dependentes);
            }
        
            $this->entityManager->commit();
            
            return $pessoa;
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    public function cadastroAutomaticoDependentes($nrorg, $nrvinculom, $dependentes) {
        try {
            
            foreach ($dependentes as $dependente) {
                $pessoaDependente = $this->getPessoaDependente($nrorg, $dependente["NRPARCDEPENDE"]);
                $this->criaDepvinculos($nrorg, $nrvinculom, $dependente["NRTIPORELACIONA"], $pessoaDependente->getNrpessoa());
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getPessoaDependente($nrorg, $nrparcdepende)
    {
        $pessoaDependente = $this->pessoaRepository->retornaPessoaPorParceiro($nrorg, $nrparcdepende);
        if (!is_object($pessoaDependente)) {
            $dtcompetencia = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true);
            $arrayPessoa = $this->pessoaService->novo($nrorg, $nrparcdepende, $dtcompetencia, null, null,
                null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null,
                null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null,
                null, null, null, null, null, null, null);
            $pessoaDependente = $arrayPessoa[0];
        }

        return $pessoaDependente;
    }

    public function criaDepvinculos($nrorg, $nrvinculom, $nrtiporelaciona, $nrpessoadep)
    {
        try {
            switch ($nrtiporelaciona) {
                case self::TIPO_RELACIONAMENTO_FILHO:
                    $this->criaDepvinculosFilho($nrorg, $nrvinculom, $nrpessoadep);
                    break;

                case self::TIPO_RELACIONAMENTO_PENSIONISTA:
                    $this->criaDepvinculosPensionista($nrorg, $nrvinculom, $nrpessoadep);
                    break;

                case self::TIPO_RELACIONAMENTO_TUTELADO:
                    $this->criaDepvinculosTutelado($nrorg, $nrvinculom, $nrpessoadep);
                    break;

                default:
                    break;
            }
        } catch (\Exception $e) {
            throw $e;
        }

    }

    /**
     * Cria os tipos de dependencias existentes para os relacionamentos do tipo "Filho (a)".
     */
    private function criaDepvinculosFilho($nrorg, $nrvinculom, $nrpessoaDep)
    {
        try {
            $this->validaIdadeMinCPF($nrpessoaDep, $nrorg);
            if ($this->validaSalarioFamilia($nrpessoaDep, $nrorg)) {
                $this->insereDepVinculo($nrorg, $nrpessoaDep, self::TIPODEPENDE_SALARIO_FAMILIA, $nrvinculom);
            }

            $this->insereDepVinculo($nrorg, $nrpessoaDep, self::TIPODEPENDE_IMPOSTO_DE_RENDA, $nrvinculom);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @todo revisar quais tipos de dependencias deverao ser criadas para este relacionamento.
     * Cria os tipos de dependencias existentes para os relacionamentos do tipo "Pensionista".
     */
    private function criaDepvinculosPensionista($nrorg, $nrvinculom, $nrpessoaDep)
    {
        try {
            $this->insereDepVinculo($nrorg, $nrpessoaDep, self::TIPODEPENDE_PENSAO_JUDICIAL, $nrvinculom);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @todo revisar quais tipos de dependencias deverao ser criadas para este relacionamento.
     * Cria os tipos de dependencias existentes para os relacionamentos do tipo "Tutelado". 
     */
    private function criaDepvinculosTutelado($nrorg, $nrvinculom, $nrpessoaDep)
    {
        try {
            $this->validaIdadeMinCPF($nrpessoaDep, $nrorg);
            $this->insereDepVinculo($nrorg, $nrpessoaDep, self::TIPODEPENDE_IMPOSTO_DE_RENDA, $nrvinculom);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Insere um novo registro na tabela fpa_depvinculo. Este metodo somente criara um novo se nao houver nenhum outro
     * fazendo a mesma dependencia.
     */
    public function insereDepVinculo($nrorg, $nrpessoaDep, $nrtipodepende, $nrvinculom)
    {
        if ($this->existeDependencia($nrorg, $nrpessoaDep, $nrtipodepende, $nrvinculom) === false) {
            try {
                $this->novo($nrorg, $nrpessoaDep, $nrtipodepende, $nrvinculom);
            } catch (\Exception $e) {
                throw $e;
            }
        }
    }

    public function validaSalarioFamilia($nrpessoaDep, $nrorg, $competencia = null)
    {
        if ($competencia == null) {
            $competencia = $this->hcmParameters->getDtCompetencia();
        }

        $pessoah = $this->pessoaRepository->retornaHistoricoAtualPorPessoa($nrorg, $nrpessoaDep, $competencia);

        if (is_object($pessoah)) {
            if ($pessoah->getDtnascpessoa()) {
                $dtnascpessoa = $pessoah->getDtnascpessoa();
                //$dtnascpessoa = DateUtil::getDataDeString($dtnascpessoa); 
                $dataAtual = DateUtil::getDataAtual();
                if ($dataAtual->diff($dtnascpessoa)->y > 14) {
                    return false;
                }
            }
        }

        return true;
    }
    
    
    public function validaIdadeMinCPF($nrpessoaDep, $nrorg, $competencia = null){
         if ($competencia == null) {
            $competencia = $this->hcmParameters->getDtCompetencia();
        }
        $pessoaDependente = $this->pessoaRepository->retornaHistoricoAtualPorPessoa($nrorg, $nrpessoaDep, $competencia);
        if ($pessoaDependente->getDtnascpessoa()){
            $dataNascDep = DateUtil::getDataDeString(substr($pessoaDependente->getDtnascpessoa(), 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
            $dataCalenda = DateUtil::getDataDeString(substr($competencia, 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
            $idadeDep = $dataCalenda->diff($dataNascDep)->y; 
            if(!$pessoaDependente->getNrcpfpessoa() && $idadeDep >= $this->idadeMinCPF )
                throw new \Exception('CPF não está cadastrado para Dependente '.$pessoaDependente->getNmpessoa(),10);
        }else {
             throw new \Exception('Data de nascimento não está cadastrada para Dependente '.$pessoaDependente->getNmpessoa(),10);
        }
        
    }
    
    public function alteraCPFDpendente($nrpessoaDep, $cpfdependente, $nrorg, $dtcompetencia = null){
        try{

            $dtcompetencia = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true);
    
            $result = $this->entityManager->getRepository(Repositories::GPE_PESSOA)->findOneBy(array("nrorg"=> $nrorg,"nrpessoa" => $nrpessoaDep));
    
            $pessoaHistorico = $this->pessoaRepository->retornaHistoricoAtualPorPessoa($nrorg,$result->getNrpessoa(),$dtcompetencia);
            
            if($cpfdependente != $pessoaHistorico->getNrcpfpessoa()){
                if($pessoaHistorico->getDtmescompetenc() == $dtcompetencia){
                    
                    $pessoaHistorico->setNrcpfpessoa($cpfdependente);

                    $this->entityManager->persist($pessoaHistorico);
                    $this->entityManager->flush();
                }
                else{
                    $this->pessoaService->novoHistorico(
                        $nrorg,$pessoaHistorico->getNrpessoa(),$dtcompetencia,$pessoaHistorico->getNmpessoa(),$pessoaHistorico->getDsapelidopess(),$pessoaHistorico->getNrcertnascpes(),$pessoaHistorico->getDtnascpessoa(),
                        $pessoaHistorico->getIdsexopessoa(),$pessoaHistorico->getNrcondfispes(),$cpfdependente,$pessoaHistorico->getNrctpspessoa(),$pessoaHistorico->getNrseriectpspes(),$pessoaHistorico->getSgufctpspes(),
                        $pessoaHistorico->getDtctpspessoa(),$pessoaHistorico->getNrpispaseppes(),$pessoaHistorico->getDtinscpispasep(),$pessoaHistorico->getNrinscinsspes(),$pessoaHistorico->getNrinscisspes(),$pessoaHistorico->getCdestacivil(),
                        $pessoaHistorico->getNrcertcasapes(),$pessoaHistorico->getDtcasapessoa(),$pessoaHistorico->getIdisentituelei(),$pessoaHistorico->getNrtitueleipes(),$pessoaHistorico->getNrsecaeleipes(),$pessoaHistorico->getNrzonaeleipes(),
                        $pessoaHistorico->getNrcarthabpes(),$pessoaHistorico->getDthabcartpes(),$pessoaHistorico->getDtvalhabcartpes(),$pessoaHistorico->getDscateghabcart(),$pessoaHistorico->getNrcatmilitarpe(),$pessoaHistorico->getNrcertresepes(),
                        $pessoaHistorico->getDscategcertrese(),$pessoaHistorico->getDtcertresepes(),$pessoaHistorico->getCdexcertresepes(),$pessoaHistorico->getNrracapessoa(),$pessoaHistorico->getIdgrupsangupes(),$pessoaHistorico->getIdfatorrhpes(),
                        $pessoaHistorico->getCdpais(),$pessoaHistorico->getSgestado(),$pessoaHistorico->getCdmunicipio(),$pessoaHistorico->getNrrgpessoa(),$pessoaHistorico->getCdexrgpessoa(),$pessoaHistorico->getSgufrgpessoa(),
                        $pessoaHistorico->getDslocalexrgpes(),$pessoaHistorico->getDtexrgpessoa(),$pessoaHistorico->getNrnacionalid(),$pessoaHistorico->getNrgrauinstr(),$pessoaHistorico->getDspronometrat() ,$pessoaHistorico->getNranochegapais() ,
                        $pessoaHistorico->getNrdocuestrang(),$pessoaHistorico->getIdpesnaturaliza(),$pessoaHistorico->getDtnaturalizapes(),$pessoaHistorico->getIdposvistoperm(),$pessoaHistorico->getDtpermavisto(),$pessoaHistorico->getNrconselhoreg(),
                        $pessoaHistorico->getNrinscrconsreg(),$pessoaHistorico->getDtexconsreg(),$pessoaHistorico->getSgufconsreg(),$pessoaHistorico->getCdtpbaixapag(),$pessoaHistorico->getNrlivrocertnasc(),
                        $pessoaHistorico->getNrfolhacertnasc(), $pessoaHistorico->getDscomplcertnasc(),$pessoaHistorico->getNrregctpspes()
                        );    
                }
            }
    
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
} 