<?php

namespace HCM\Geral\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Geral\Services\IncidenciaSindicatoService;
use Zeedhi\Framework\DataSource\DataSet;

class IncidenciaSindicatoController extends Crud {

    protected $dataSourceName_set = 'incidenciasind_set';
    protected $dataSourceName_get = 'incidenciasind_get';

    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, IncidenciaSindicatoService $incidenciaSindicatoService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->incidenciaSindicatoService = $incidenciaSindicatoService;
        parent::__construct($dataSourceManager);
    }

    public function getIncidenciaSindicato(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceName_get;
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $rows = $dataSet->getRows();
        foreach($rows as $key => &$row){
            if(isset($row['DSNRTIPOINCIDE'])){
                $row['DSNRTIPOINCIDE'] = explode(",", $row['DSNRTIPOINCIDE']);
                
                foreach($row['DSNRTIPOINCIDE'] as $chave => $value)
                    $row['DSNRTIPOINCIDE'][$chave] = intval($value);
                    
                $row['NMTIPOINCIDE'] = explode(",", $row['NMTIPOINCIDE']);
            }    
        }
        $response->addDataSet(new DataSet($this->dataSourceName, $rows));
    }
    
    public function setIncidenciaSindicato(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceName_set;
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRINCIDSIND', $row)) {
                    if ($row['NRINCIDSIND'] === '') {
                        $row['NRINCIDSIND'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'INCIDENCIASIND', 1, 12, true); 
                    }
                } else {
                    $row['NRINCIDSIND'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'INCIDENCIASIND', 1, 12, true); 
                }
                
                $this->incidenciaSindicatoService->validaCamposVazios($row);
                
                if( isset($row['DSNRTIPOINCIDE']) && is_array($row['DSNRTIPOINCIDE']) )
                    $row['DSNRTIPOINCIDE'] = implode(",", $row['DSNRTIPOINCIDE']);
                
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteIncidenciaSindicato(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = $this->dataSourceName_set;
        try{
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 