<?php

namespace HCM\Geral\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\DateUtil;

class PrioridadesController extends Crud {

    protected $dataSourceGetPrioridades = 'getPrioridades';
    protected $dataSourceName           = 'desPrioridades_setPrioridades';

    protected $parameterBagColumns = array('NRORG');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters   = $hcmParameters;
        parent::__construct($dataSourceManager);
    }

    public function getPrioridades (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceGetPrioridades);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }

    public function setPrioridade (DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            foreach($request->getDataSet()->getRows() as $row) {
                if($row['__is_new'] === true){

                    $row['DTINCLUSAO'] = $row['DTATUAL'];
                    $row['CDOPERINCLUSAO'] = $row['CDOPERLOG'];
                    $row['NRORGINCLUSAO'] = $row['NRORGTRAB'];
                    $row['DTULTATU'] = $row['DTATUAL'];
                    $row['CDOPERULTATU'] = $row['CDOPERLOG'];
                    $row['NRORGULTATU'] = $row['NRORGTRAB'];
                    $row['NRPRIORIDADE'] = \HCM\Util\NovoCodigo::geraCodigo( $row['NRORGTRAB'], 'DES_PRIORIDADE', 1, 12, true);

                } else {

                    $row['DTULTATU'] = $row['DTATUAL'];
                    $row['CDOPERULTATU'] = $row['CDOPERLOG'];
                    $row['NRORGULTATU'] = $row['NRORGTRAB'];
                }
                $row['NRORG'] = $row['NRORGTRAB'];

                if(!array_key_exists('IDATIVO', $row))
                    $row['IDATIVO'] = 'S';
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deletePrioridade(DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            parent::delete($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
}