<?php
/**
 * Created by PhpStorm.
 * User: leonardocarvalho
 * Date: 02/03/2016
 * Time: 16:57
 */

namespace HCM\Geral\Services;

use Doctrine\ORM\EntityManager;
use HCM\Util\DateUtil;
use Teknisa\Libs\Util\Environment;
use HCM\Util\HCMParameters;

class ReferencesGeraisServices {

    protected $entityManager;
    /** @var Environment */
    protected $environment;
    protected $hcmParameters;

    public function __construct(EntityManager $entityManager, Environment $environment, HCMParameters $hcmParameters) {
        $this->entityManager = $entityManager;
        $this->environment   = $environment;
        $this->hcmParameters = $hcmParameters;
    }

    public function getEstadocivil($parameters) {

        //$nrorg = $this->environment->getNrOrgTrab();
        $nrorg = 0;

        $query = $this->entityManager->getConnection()->prepare("
            SELECT
              CDESTACIVIL,
              NMESTACIVIL
              FROM ESTADOCIVIL
             WHERE NRORG = :P_NRORG
        ");

        $query->execute(array(
            "P_NRORG" => $nrorg
        ));

        $result = $query->fetchAll();
        return $result;
    }

    public function getNacionalidade($parameters) {

        //$nrorg = $this->environment->getNrOrgTrab();
        $nrorg = 0;

        $query = $this->entityManager->getConnection()->prepare("
            SELECT
              NRNACIONALIDADE,
              DSNACIONALIDADE,
              CDRAIS,
              CDPAIS
              FROM GPE_NACIONALIDADE
             WHERE NRORG = :P_NRORG
             ORDER BY DSNACIONALIDADE
        ");

        $query->execute(array(
            "P_NRORG" => $nrorg
        ));

        $result = $query->fetchAll();
        return $result;
    }

    public function getSimNao() {

        $query = $this->entityManager->getConnection()->prepare("
            SELECT 'S' CD, 'Sim' DS FROM DUAL
            UNION
            SELECT 'N' CD, 'Não' DS FROM DUAL
        ");

        $query->execute();

        $result = $query->fetchAll();
        return $result;
    }

    public function getGrauintrucao() {

        //$nrorg = $this->environment->getNrOrgTrab();
        $nrorg = 0;

        $query = $this->entityManager->getConnection()->prepare("
            SELECT
              NRGRAUINSTR,
              NMGRAUINSTR,
              NMABREV,
              CDESOCIAL
              FROM GPE_GRAUINSTR
             WHERE NRORG = :P_NRORG
             ORDER BY NRGRAUINSTR
        ");

        $query->execute(array(
            "P_NRORG" => $nrorg
        ));

        $result = $query->fetchAll();
        return $result;
    }

    public function getFormaComnunicacao() {

        //$nrorg = $this->environment->getNrOrgTrab();
        $nrorg = 0;

        $query = $this->entityManager->getConnection()->prepare("
            SELECT
              CDFORMACOMU,
              NMFORMACOMU
              FROM FORMACOMU
             WHERE NRORG = :P_NRORG
             ORDER BY NMFORMACOMU
        ");

        $query->execute(array(
            "P_NRORG" => $nrorg
        ));

        $result = $query->fetchAll();
        return $result;
    }

    public function getPais($conditions) {

        //$nrorg = $this->environment->getNrOrgTrab();
        $nrorg = 0;

        $query = $this->entityManager->getConnection()->prepare("
            SELECT * 
              FROM (SELECT DECODE(CDPAIS,'0055','AAA',NMPAIS) AS NMORDER,
                           CDPAIS,
                           NMPAIS
                      FROM PAIS
                     WHERE NRORG = :P_NRORG
                     ORDER BY NMORDER, NMPAIS) 
             WHERE 1 = 1
            $conditions
        ");

        $query->execute(array(
            "P_NRORG" => $nrorg
        ));

        $result = $query->fetchAll();
        return $result;
    }

    public function getMunicipio() {

        //$nrorg = $this->environment->getNrOrgTrab();
        $nrorg = 0;

        $query = $this->entityManager->getConnection()->prepare("
            SELECT
              CDMUNICIPIO,
              INITCAP(NMMUNICIPIO) NMMUNICIPIO,
              CDMUNICIBGE
              FROM MUNICIPIO
             WHERE NRORG   = :P_NRORG
               AND IDATIVO = 'S'
             ORDER BY NMMUNICIPIO
        ");

        $query->execute(array(
            "P_NRORG" => $nrorg
        ));

        $result = $query->fetchAll();
        return $result;
    }

    public function getOrganizacoesOper() {

        $cdoperador = $this->environment->getCdoperador();

        $query = $this->entityManager->getConnection()->prepare("
            SELECT
              OP.NRORGREL AS NRORGTRAB,
              ORG.NMORGANIZACAO
            FROM ORGANIZACAOOPER OP
            JOIN ORGANIZACAO ORG
              ON ORG.NRORG = OP.NRORGREL
           WHERE OP.CDOPERADOR = :P_CDOPERADOR
           ORDER BY ORG.NMORGANIZACAO
        ");

        $query->execute(array(
            "P_CDOPERADOR" => $cdoperador
        ));

        $result = $query->fetchAll();
        return $result;
    }

    public function getEstruturasOper() {

        $cdoperador             = $this->environment->getCdoperador();
        $nrtpestruturalegal     = $this->hcmParameters->getTipoEstruturaLegalOrg();
        $nrtpestruturagerencial = $this->hcmParameters->getTipoEstruturaGerencialOrg();
        $dtmescompetencia       = $this->hcmParameters->getDtCompetencia();
        $nrorg                  = $this->hcmParameters->getNrorgTrabalho();

        $query = $this->entityManager->getConnection()->prepare("
            SELECT ESTRUTURAM.NRESTRUTURAM, ESTRUTURAH.NMESTRUTURAH, 
            ESTRUTURAM.NRORG, ESTRUTURAM.NRTIPOESTRUTURA, TIPOESTRUTURA.NMTIPOESTRUTURA
            FROM ESTRUTURAM

            INNER JOIN ESTRUTURAH
            ON ESTRUTURAH.NRESTRUTURAM = ESTRUTURAM.NRESTRUTURAM
            AND ESTRUTURAH.NRORG = ESTRUTURAM.NRORG

            INNER JOIN  
               (SELECT MAX(DTMESCOMPETENC) DTMESCOMPETENC, NRORG, NRESTRUTURAM
                FROM ESTRUTURAH
                WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENCIA
                GROUP BY NRORG, NRESTRUTURAM
               ) EHATUAL
            ON EHATUAL.DTMESCOMPETENC = ESTRUTURAH.DTMESCOMPETENC
            AND EHATUAL.NRORG = ESTRUTURAH.NRORG
            AND EHATUAL.NRESTRUTURAM = ESTRUTURAH.NRESTRUTURAM
              
            LEFT OUTER JOIN TIPOESTRUTURA
              ON TIPOESTRUTURA.NRTIPOESTRUTURA = ESTRUTURAM.NRTIPOESTRUTURA
            WHERE (ESTRUTURAM.NRTIPOESTRUTURA = :P_NRTPESTRUTURALEGAL OR ESTRUTURAM.NRTIPOESTRUTURA = :P_NRESTRUTURAGEREN)
            AND ESTRUTURAM.NRORG = :P_NRORG
            AND EXISTS (SELECT 1
                          FROM ESTRUTURAOPER, ORGANIZACAOOPER
                         WHERE ESTRUTURAOPER.NRESTRUTURAMREL = ESTRUTURAM.NRESTRUTURAM
                           AND ESTRUTURAOPER.NRORG           = :P_NRORG
                           AND ESTRUTURAOPER.NRORGANIZACAOOPER = ORGANIZACAOOPER.NRORGANIZACAOOPER
                           AND ORGANIZACAOOPER.CDOPERADOR      = :P_CDOPERADOR)
            ORDER BY ESTRUTURAH.NMESTRUTURAH
        ");

        $query->execute(array(
            "P_NRTPESTRUTURALEGAL" => $nrtpestruturalegal,
            "P_NRESTRUTURAGEREN"   => $nrtpestruturagerencial,
            "P_DTMESCOMPETENCIA"   => $dtmescompetencia,
            "P_CDOPERADOR"         => $cdoperador,
            "P_NRORG"              => $nrorg
        ));

        $result = $query->fetchAll();
        return $result;
    }

}