<?php

namespace HCM\PTO\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use HCM\PTO\Services\MarcacaoPontoService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\DateUtil;

class MarcacaoPontoController extends Crud {

    protected $parameterBagColumns = array('P_NRORG' , 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC', 'NRPERIODOAPURACAO');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, MarcacaoPontoService $marcacaoPontoService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters   = $hcmParameters;
        $this->marcacaoPontoService = $marcacaoPontoService;
        parent::__construct($dataSourceManager);
    }

    public function getMarcacaoPonto (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $this->dataSourceName = "getMarcacaoPonto";
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceName);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function setMarcacaoPonto (DTO\Request\DataSet $request, DTO\Response $response) {
        
        try{
            $this->dataSourceName = "GpeMarcacaoponto_setMarcacaoPonto";
            foreach($request->getDataSet()->getRows() as &$row) {
                if(array_key_exists('NRMARCACAOPONTO', $row)){
                    if(empty($row['NRMARCACAOPONTO'])){
                        $row['NRMARCACAOPONTO'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'GPE_MARCACAOPONTO', 1, 12, true);
                    }
                } else {
                    $row['NRMARCACAOPONTO'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'GPE_MARCACAOPONTO', 1, 12, true);
                }
                if(array_key_exists('IDATIVO', $row)){
                    if($row['IDATIVO'] === ''){
                    
                    $row['IDATIVO'] = 'S';
                    }
                } else {
                    $row['IDATIVO'] = 'S';
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
     public function delete(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = "GpeMarcacaoponto_setMarcacaoPonto";
            parent::delete($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function importaArquivoParaMarcacao(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $row = $request->getDataSet()->getRows();
            $row = $row[0];
            $row['NRORG'] = $this->hcmParameters->getNrorgTrabalho();
            $row['NRORG_PADRAO'] = $this->hcmParameters->getNrorgPadrao();
            $row['MESCOMPETENCIA'] = $this->hcmParameters->getDtCompetencia();
            foreach ($row['DSPATH'] as $arquivo) {
                $row['DSPATH'] = [$arquivo];
                $mensagem = $this->marcacaoPontoService->importaArquivoParaMarcacao($row);    
            }
            if(array_key_exists('erro', $mensagem)){
                $message = new \Zeedhi\Framework\DTO\Response\Message($mensagem['erro'], 'E');
                $response->addMessage($message);
            }
            if($mensagem['sucesso']){
                $message = new \Zeedhi\Framework\DTO\Response\Message($mensagem['sucesso'], 'S');
                $response->addMessage($message);
            }
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $this->parameterBag->set('NRORG', $this->hcmParameters->getNrorgTrabalho());  
    

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}