<?php

namespace HCM\PTO\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\DateUtil;

class OcorrenciaFrequenciaController extends Crud {

    protected $parameterBagColumns = array('NRORG', 'DTMESCOMPETENC', 'NRORGPADRAO', 'NRPERIODOAPURACAO', 'NRHORDIAM', "P_DTMESCOMPETENC");
    protected $periodoApuracao;
    protected $marcacaoPontoService;
    protected $codigoapuracao;
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters,\HCM\PTO\Services\OcorrenciaFrequenciaService $ocorrenciaFrequenciaService, \HCM\PTO\Services\PeriodoApuracaoService $periodoApuracao,\HCM\PTO\Services\MarcacaoPontoService $marcacaoPontoService){
        $this->periodoApuracao = $periodoApuracao;
        $this->parameterBag = $parameterBag;
        $this->hcmParameters   = $hcmParameters;
        $this->ocorrenciaFrequenciaService = $ocorrenciaFrequenciaService;
        $this->marcacaoPontoService = $marcacaoPontoService;
        parent::__construct($dataSourceManager);
    }

    public function getOcorrencia (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $conditions = $request->getFilterCriteria()->getConditions();
            $parameters = array();
            foreach ($conditions as $filter) {
                foreach ($filter as $key => $value) {
                    if ($key == 'columnName') {
                        $parameters[$value] = $filter['value'];
                        unset($filter['columnName']);
                    }
                }
            }
            $results = $this->periodoApuracao->getPeriodoApuracaoAtual($parameters, $conditions);
            $this->codigoapuracao = array();
            foreach($results[0] as $index => $result){
                $this->codigoapuracao[$index] = $result["NRPERIODOAPURACAO"];
            }
            $filterData = $request->getFilterCriteria();
            $filterData->addCondition('NRPERIODOAPURACAO', $this->codigoapuracao, '=');
            $filterData->setDataSourceName($this->dataSourceName = "getOcorrenciaFrequencia");
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getMarcacoesPonto($request, $response) {
        try {
            $filters = $request->getFilterCriteria()->getConditions();
            $parameters = array();
            foreach ($filters as $filter) {
                unset($filter['operator']);
                foreach ($filter as $key => $value) {
                    if ($key == 'columnName') {
                        $parameters[$value] = $filter['value'];
                        unset($filter['columnName']);
                    }
                }
            }
            $marcacaoPontos = $this->marcacaoPontoService->getMarcacoesPonto($parameters);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getMarcacoesPonto', array($marcacaoPontos)));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getHoradiah (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceName = "gpeHordiah_getHoradiah");
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getSaida (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceName = "gpeIntervalosHorario_getIntervaloHorarioSaida");
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {
        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $this->parameterBag->set('NRPERIODOAPURACAO', $this->codigoapuracao);
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
    
    public function setOcorrencia(DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            $this->dataSourceName = "ptoProgocorrencia_setProgocorrencia";
            foreach($request->getDataSet()->getRows() as &$row) {
                if(array_key_exists('NRPROGOCORRENCIA', $row)){
                    if(empty($row['NRPROGOCORRENCIA'])){
                        $row['NRPROGOCORRENCIA'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'PTO_PROGOCORRENCIA', 1, 12, true);
                    }
                } else {
                    $row['NRPROGOCORRENCIA'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'PTO_PROGOCORRENCIA', 1, 12, true);
                }
                if(!array_key_exists('NRMOTIVOOCORFREQ', $row)){
                    $row['NRMOTIVOOCORFREQ'] = null;
                }
                if(!array_key_exists('NRHORDIAM', $row)){
                    $row['NRHORDIAM'] = null;
                }
                if(!array_key_exists('NRINTERVALOHORARIO', $row)){
                    $row['NRINTERVALOHORARIO'] = null;
                }
                 if(!array_key_exists('NRMARCACAOPONTO', $row)){
                    $row['NRMARCACAOPONTO'] = null;
                }
                if(!array_key_exists('NRSITUFREQUENCIA', $row)){
                    $row['NRSITUFREQUENCIA'] = null;
                }
                if(!array_key_exists('NRSITUFREQGATILHO', $row)){
                    $row['NRSITUFREQGATILHO'] = null;
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
     public function delete(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = "ptoProgocorrencia_setProgocorrencia";
            parent::delete($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function alteraStatusOcorrencia(DTO\Request\Row $request, DTO\Response $response){
        try{
            $row = $request->getRow();
            
            $nrprogocorrencia       = $row['NRPROGOCORRENCIA'];
            $idprogocorconfirmada   = $row['IDPROGOCORCONFIRMADA'];
            $dsobservacaorecusa     = $row['DSOBSERVACAORECUSA'];
            
            $statusAlteradoComSucesso = $this->ocorrenciaFrequenciaService->alteraStatusOcorrencia($nrprogocorrencia, $idprogocorconfirmada, $dsobservacaorecusa);
            
            $dataSet = array('success' => $statusAlteradoComSucesso);
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('alteraStatusOcorrencia', $dataSet));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function downloadArquivosOcorrencia(DTO\Request\Row $request, DTO\Response $response){
        try{
            $row = $request->getRow();
            
            $nrprogocorrencia = $row['NRPROGOCORRENCIA'];
            
            $arquivoOcorrenciaNome = $this->ocorrenciaFrequenciaService->downloadArquivosOcorrencia($nrprogocorrencia);
            
            $dataSet = array('FILENAME' => $arquivoOcorrenciaNome);
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('downloadArquivosOcorrencia', $dataSet));
        } catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }
}